/**********************************************************************
**
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Designer.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "classbrowserinterfaceimpl.h"
#include "designerinterface.h"
#include "objectbrowser.h"

ClassBrowserInterfaceImpl::ClassBrowserInterfaceImpl( QUnknownInterface *outer )
    : parent( outer ),
      ref( 0 ), m_receiver(0), m_slot(0)
{
}

ClassBrowserInterfaceImpl::~ClassBrowserInterfaceImpl()
{
    qDebug( "!!!!! class browser iface destroyed !!!!!" );
}

ulong ClassBrowserInterfaceImpl::addRef()
{
    return parent ? parent->addRef() : ref++;
}

ulong ClassBrowserInterfaceImpl::release()
{
    if ( parent )
	return parent->release();
    if ( !--ref ) {
	delete this;
	return 0;
    }
    return ref;
}

QRESULT ClassBrowserInterfaceImpl::queryInterface( const QUuid &uuid, QUnknownInterface** iface )
{
    if ( parent )
	return parent->queryInterface( uuid, iface );

    *iface = 0;
    if ( uuid == IID_QUnknown )
	*iface = (QUnknownInterface*)this;
    else if ( uuid == IID_ClassBrowser )
	*iface = (ClassBrowserInterface*)this;
    else
	return QE_NOINTERFACE;

    (*iface)->addRef();
    return QS_OK;
}

QListView *ClassBrowserInterfaceImpl::createClassBrowser( QWidget *p ) const
{
    return new PerlQtObjectBrowser( p, "perleditor_objectbrowser", parent );
}

void ClassBrowserInterfaceImpl::update( const QString &code ) const
{
   if( m_ob )
       m_ob->update( code );
}

void ClassBrowserInterfaceImpl::clear() const
{

}

void ClassBrowserInterfaceImpl::onClick( QObject *receiver, const char *slot )
{
    m_receiver= receiver;
    m_slot = slot;
}

