/**********************************************************************
**
** Copyright (C) 2003 Germain Garand <germain@ebooksfrance.org>
**
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**********************************************************************/

#ifndef PERLQTOBJECTBROWSER_H
#define PERLQTOBJECTBROWSER_H

#include <qlistview.h>
#include <qpixmap.h>
#include <qguardedptr.h>
#include "languageinterface.h"

class CommonInterface;
class EditorInterfaceImpl;
struct DesignerInterface;
struct DesignerFormWindow;
struct QUnknownInterface;
class PerlEditor;

class PerlHierarchyItem : public QListViewItem
{
public:
    enum Type {
	Widget,
	SlotParent,
	SlotPublic,
	SlotProtected,
	SlotPrivate,
	Slot,
	DefinitionParent,
	Definition,
	Event,
	EventFunction,
	FunctParent,
	FunctPublic,
	FunctProtected,
	FunctPrivate,
	Function,
	VarParent,
	VarPublic,
	VarProtected,
	VarPrivate,
	Variable
    };

    PerlHierarchyItem( Type type, QListViewItem *parent, QListViewItem *after,
		   const QString &txt1, const QString &txt2, const QString &txt3 );
    PerlHierarchyItem( Type type, QListView *parent, QListViewItem *after,
		   const QString &txt1, const QString &txt2, const QString &txt3 );

    virtual void paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int align );
    void updateBackColor();

    void setObject( QObject *o );
    QObject *object() const;

    void setText( int col, const QString &txt ) { if ( !txt.isEmpty() ) QListViewItem::setText( col, txt ); }

    int rtti() const { return (int)typ; }

private:
    void okRename( int col );
    void cancelRename( int col );

private:
    QColor backgroundColor();
    QColor backColor;
    QObject *obj;
    Type typ;

};


class PerlHierarchyList : public QListView
{
    Q_OBJECT

public:
    PerlHierarchyList( QWidget *parent, const char *name, bool doConnects = TRUE );

    virtual void setup() = 0;
    void setOpen( QListViewItem *i, bool b );
    void changeNameOf( QObject *o, const QString &name );
    void setFormWindow( DesignerFormWindow *fw ) { formWindow = fw; }
    void drawContentsOffset( QPainter *p, int ox, int oy,
			     int cx, int cy, int cw, int ch ) {
	setUpdatesEnabled( FALSE );
	triggerUpdate();
	setUpdatesEnabled( TRUE );
	QListView::drawContentsOffset( p, ox, oy, cx, cy, cw, ch );
    }

    void insertEntry( QListViewItem *i, const QPixmap &pix = QPixmap(), const QString &s = QString::null );

protected:
    void keyPressEvent( QKeyEvent *e );
    void keyReleaseEvent( QKeyEvent *e );
    void viewportMousePressEvent( QMouseEvent *e );
    void viewportMouseReleaseEvent( QMouseEvent *e );

private:
    QObject *findObject( QListViewItem *i );
    QListViewItem *findItem( QObject *o );

private slots:
    virtual void objectClicked( QListViewItem *i );
    virtual void objectDoubleClicked( QListViewItem *i );
    virtual void showRMBMenu( QListViewItem *, const QPoint & );

protected:
    DesignerFormWindow *formWindow;
    bool deselect;

};


class PerlQtObjectBrowser : public PerlHierarchyList
{
    Q_OBJECT

public:
    PerlQtObjectBrowser( QWidget * parent = 0 , const char * name = 0, QUnknownInterface* outer = 0);
    void setup();
    void setupVariables();
    void refresh( const QString& code );
    virtual ~PerlQtObjectBrowser();

signals:
    virtual void jumpTo( const QString &, const QString &, int );

public slots:
    void update( const QString& code );

protected slots:
    void contentsMouseDoubleClickEvent( QMouseEvent *e );
    void showRMBMenu( QListViewItem *, const QPoint & );
    void formChanged();
    void setupJump();

private slots:
    virtual void objectClicked( QListViewItem *i );
    void updateFunctionList();
    void renamed( QListViewItem *i );

private:
    void execFunctionDialog();
    void save( QListViewItem *p, QListViewItem *i );
    void editDefinition( const QString &def );

    QUnknownInterface *iface;
    DesignerInterface *dIface;
    QValueList<LanguageInterface::Function> functionList;
    bool popupOpen;
    bool isform;
    QString curtext;
    PerlHierarchyItem *itemSlots, *itemPrivate, *itemProtected, *itemPublic;
    PerlHierarchyItem *itemFunct, *itemFunctPriv, *itemFunctProt, *itemFunctPubl;
    PerlHierarchyItem *itemVar, *itemVarPriv, *itemVarProt, *itemVarPubl;

};


#endif
