/**********************************************************************
**
** Copyright (C) 2003 Germain Garand <germain@ebooksfrance.org>
**
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**********************************************************************/

#include "pqtapiprocess.h"
#include <qstringlist.h>
#include <qapplication.h>
#include "conf.h"

PqtApiProcess* PqtApiProcess::pqtapi = 0;
bool PqtApiProcess::canrun = TRUE;

PqtApiProcess* PqtApiProcess::instance()
{
    if( !pqtapi )
    {
        QStringList lst;
	QString path;
	path = Config::pqtapiPath( "/Trolltech/PerlEditor" );
        lst << path << "-m"; //- TODO: take path from global config
        pqtapi = new PqtApiProcess( lst );
	if( !pqtapi )
	   return 0;
        pqtapi->setCommunication( QProcess::Stdin | QProcess::Stdout );
        connect( pqtapi, SIGNAL(readyReadStdout()),
                pqtapi, SLOT(readFuncList()) );
        if(!pqtapi->start())
        {
            delete pqtapi;
            pqtapi = 0;
	    canrun = false;
        }
    }
    return pqtapi;
}

PqtApiProcess::PqtApiProcess( const QStringList &lst ): QProcess( lst ),
	m_gotstart(FALSE), m_finished(TRUE)
{
    m_cache.setAutoDelete(TRUE);
}

PqtApiProcess::~PqtApiProcess()
{
    if( isRunning() )
    {
        tryTerminate();
    }
}

void PqtApiProcess::readFuncList()
{
    while ( canReadLineStdout() )
    {
        QString f = readLineStdout();
	if( !m_gotstart)
	{
	    if( f == "__START__" )
	        m_gotstart = true;
	    continue;
	}
        else if( f == "__END__" )
        {
            m_finished = true;
            return;
        }

        m_funclist->append( f );
    }
}

bool PqtApiProcess::startFuncList()
{
    m_gotstart = false;
    m_finished = false;
    m_funclist = new QStringList;
    return !!(m_funclist);
}

QStringList* PqtApiProcess::request( const QString& classname )
{
    QStringList* list = 0;
    list = m_cache.find( classname.latin1() );
    if( list || !isRunning() )
        return list;
    else if( m_gotstart && !m_finished ) // we didn't get the previous class entirely
    {
        readFuncList(); // so let's give it a last chance to complete
	if( !m_finished ) // no luck? Oh, well.
	{
	    m_finished = true;
	}
    }
    if( !startFuncList() )
        return 0;
    m_cache.insert( classname.latin1(), m_funclist );
    QString req( classname);
    req.append("\n");
    writeToStdin( req );
    qApp->processEvents();
    if( m_gotstart && m_finished && m_funclist->isEmpty())
    {
        m_cache.remove( classname );
	return list;
    }
    return m_funclist;
}
